<?php
/**
 * Categories view
 *
 * @package daext-autolinks-manager
 */

if ( ! current_user_can( 'manage_options' ) ) {
	wp_die( esc_html__( 'You do not have sufficient permissions to access this page.', 'daext-autolinks-manager' ) );
}

?>

<!-- process data -->

<?php

// Initialize variables -----------------------------------------------------------------------------------------------.
$dismissible_notice_a = array();

// Preliminary operations ---------------------------------------------------------------------------------------------.
global $wpdb;

// Actions.
$data['edit_id']        = isset( $_GET['edit_id'] ) ? intval( $_GET['edit_id'], 10 ) : null;
$data['delete_id']      = isset( $_POST['delete_id'] ) ? intval( $_POST['delete_id'], 10 ) : null;
$data['clone_id']       = isset( $_POST['clone_id'] ) ? intval( $_POST['clone_id'], 10 ) : null;
$data['update_id']      = isset( $_POST['update_id'] ) ? intval( $_POST['update_id'], 10 ) : null;
$data['form_submitted'] = isset( $_POST['form_submitted'] ) ? intval( $_POST['form_submitted'], 10 ) : null;

// Filter and search data.
$data['s'] = isset( $_GET['s'] ) ? sanitize_text_field( wp_unslash( $_GET['s'] ) ) : null;

// Form data.
if ( ! is_null( $data['update_id'] ) || ! is_null( $data['form_submitted'] ) ) {

	// Sanitization ---------------------------------------------------------------------------------------------------.
	$name        = isset( $_POST['name'] ) ? sanitize_text_field( wp_unslash( $_POST['name'] ) ) : null;
	$description = isset( $_POST['description'] ) ? sanitize_text_field( wp_unslash( $_POST['description'] ) ) : null;

	// Validation -----------------------------------------------------------------------------------------------------.

	// validation on "name".
	if ( mb_strlen( trim( $name ) ) === 0 || mb_strlen( trim( $name ) ) > 100 ) {
		$dismissible_notice_a[] = array(
			'message' => __( 'Please enter a valid value in the "Name" field.', 'daext-autolinks-manager' ),
			'class'   => 'error',
		);
		$invalid_data           = true;
	}

	// validation on "description".
	if ( mb_strlen( trim( $description ) ) === 0 || mb_strlen( trim( $description ) ) > 255 ) {
		$dismissible_notice_a[] = array(
			'message' => __( 'Please enter a valid value in the "Description" field.', 'daext-autolinks-manager' ),
			'class'   => 'error',
		);
		$invalid_data           = true;
	}
}

// update ---------------------------------------------------------------.
if ( ! is_null( $data['update_id'] ) && ! isset( $invalid_data ) ) {

	// Nonce verification.
	check_admin_referer( 'daextam_create_update_category', 'daextam_create_update_category_nonce' );

		// Update.

		$table_name = $wpdb->prefix . $this->shared->get( 'slug' ) . '_category';
		$safe_sql   = $wpdb->prepare(
			"UPDATE $table_name SET 
                name = %s,
                description = %s
                WHERE category_id = %d",
			$name,
			$description,
			$data['update_id']
		);

		$query_result = $wpdb->query( $safe_sql );

	if ( false !== $query_result ) {
		$dismissible_notice_a[] = array(
			'message' => __( 'The category has been successfully added.', 'daext-autolinks-manager' ),
			'class'   => 'updated',
		);
	}
} else {

	// add ------------------------------------------------------------------.
	if ( ! is_null( $data['form_submitted'] ) && ! isset( $invalid_data ) ) {

		// Add.

		// Nonce verification.
		check_admin_referer( 'daextam_create_update_category', 'daextam_create_update_category_nonce' );

		$table_name = $wpdb->prefix . $this->shared->get( 'slug' ) . '_category';
		$safe_sql   = $wpdb->prepare(
			"INSERT INTO $table_name SET 
            name = %s,
            description = %s",
			$name,
			$description
		);

		$query_result = $wpdb->query( $safe_sql );

		if ( false !== $query_result ) {
			$dismissible_notice_a[] = array(
				'message' => __( 'The category has been successfully added.', 'daext-autolinks-manager' ),
				'class'   => 'updated',
			);
		}
	}
}

// delete a category.
if ( ! is_null( $data['delete_id'] ) ) {

	// Nonce verification.
	check_admin_referer( 'daextam_delete_category_' . $data['delete_id'], 'daextam_delete_category_nonce' );

	// prevent deletion if the category is associated with an autolink.
	if ( $this->shared->category_is_used( $data['delete_id'] ) ) {

		$process_data_message = '<div class="error settings-error notice is-dismissible below-h2"><p>' . esc_html__(
			"This category is associated with one or more autolinks and can't be deleted.",
			'daext-autolinks-manager'
		) . '</p></div>';

		$dismissible_notice_a[] = array(
			'message' => __( 'This category is associated with one or more autolinks and can\'t be deleted.', 'daext-autolinks-manager' ),
			'class'   => 'error',
		);


	} else {

		$table_name   = $wpdb->prefix . $this->shared->get( 'slug' ) . '_category';
		$safe_sql     = $wpdb->prepare( "DELETE FROM $table_name WHERE category_id = %d ", $data['delete_id'] );
		$query_result = $wpdb->query( $safe_sql );

		if ( false !== $query_result ) {

			$dismissible_notice_a[] = array(
				'message' => __( 'The category has been successfully deleted.', 'daext-autolinks-manager' ),
				'class'   => 'updated',
			);

		}
	}
}

// clone the category.
if ( ! is_null( $data['clone_id'] ) ) {

	// Nonce verification.
	check_admin_referer( 'daextam_clone_category_' . $data['clone_id'], 'daextam_clone_category_nonce' );

	// clone the category.
	$table_name = $wpdb->prefix . $this->shared->get( 'slug' ) . '_category';
	$wpdb->query( "CREATE TEMPORARY TABLE daextam_temporary_table SELECT * FROM $table_name WHERE category_id = " . $data['clone_id'] );
	$wpdb->query( 'UPDATE daextam_temporary_table SET category_id = NULL' );
	$wpdb->query( "INSERT INTO $table_name SELECT * FROM daextam_temporary_table" );
	$wpdb->query( 'DROP TEMPORARY TABLE IF EXISTS daextam_temporary_table' );

}

// get the category data.
if ( ! is_null( $data['edit_id'] ) ) {
	$table_name   = $wpdb->prefix . $this->shared->get( 'slug' ) . '_category';
	$safe_sql     = $wpdb->prepare( "SELECT * FROM $table_name WHERE category_id = %d ", $data['edit_id'] );
	$category_obj = $wpdb->get_row( $safe_sql );
}

?>

<!-- output -->

<div class="wrap">

	<div id="daext-header-wrapper" class="daext-clearfix">

		<h2><?php esc_html_e( 'Autolinks Manager - Categories', 'daext-autolinks-manager' ); ?></h2>

		<form action="admin.php" method="get" id="daext-search-form">

			<input type="hidden" name="page" value="daextam-categories">

			<p><?php esc_html_e( 'Perform your Search', 'daext-autolinks-manager' ); ?></p>

			<?php

			if ( ! is_null( $data['s'] ) ) {
				if ( mb_strlen( trim( $data['s'] ) ) > 0 ) {
					$search_string = $data['s'];
				} else {
					$search_string = '';
				}
			} else {
				$search_string = '';
			}

			?>

			<input type="text" name="s"
					value="<?php echo esc_attr( stripslashes( $search_string ) ); ?>" autocomplete="off" maxlength="255">
			<input type="submit" value="">

		</form>

	</div>

	<div id="daext-menu-wrapper">

		<?php $this->dismissible_notice( $dismissible_notice_a ); ?>

		<!-- table -->

		<?php

		$filter = '';

		// create the query part used to filter the results when a search is performed.
		if ( ! is_null( $data['s'] ) ) {

			if ( mb_strlen( trim( $data['s'] ) ) > 0 ) {
				$filter = $wpdb->prepare(
					'WHERE (name LIKE %s OR description LIKE %s)',
					'%' . $data['s'] . '%',
					'%' . $data['s'] . '%'
				);
			}
		}

		// retrieve the total number of categories.
		$table_name  = $wpdb->prefix . $this->shared->get( 'slug' ) . '_category';
		$total_items = $wpdb->get_var( "SELECT COUNT(*) FROM $table_name $filter" );

		// Initialize the pagination class.
		require_once $this->shared->get( 'dir' ) . '/admin/inc/class-daextam-pagination.php';
		$pag = new daextam_pagination();
		$pag->set_total_items( $total_items );// Set the total number of items.
		$pag->set_record_per_page( 10 ); // Set records per page.
		$pag->set_target_page( 'admin.php?page=' . $this->shared->get( 'slug' ) . '-categories' );// Set target page.
		$pag->set_current_page();// set the current page number.

		?>

		<!-- Query the database -->
		<?php
		$query_limit = $pag->query_limit();
		$results     = $wpdb->get_results(
			"SELECT * FROM $table_name $filter ORDER BY category_id DESC $query_limit",
			ARRAY_A
		);
		?>

		<?php if ( count( $results ) > 0 ) : ?>

			<div class="daext-items-container">

				<!-- list of tables -->
				<table class="daext-items">
					<thead>
					<tr>
						<th>
							<div><?php esc_html_e( 'Category ID', 'daext-autolinks-manager' ); ?></div>
							<div class="help-icon"
								title="<?php esc_attr_e( 'The ID of the category.', 'daext-autolinks-manager' ); ?>"></div>
						</th>
						<th>
							<div><?php esc_html_e( 'Name', 'daext-autolinks-manager' ); ?></div>
							<div class="help-icon"
								title="<?php esc_attr_e( 'The name of the category.', 'daext-autolinks-manager' ); ?>"></div>
						</th>
						<th>
							<div><?php esc_html_e( 'Description', 'daext-autolinks-manager' ); ?></div>
							<div class="help-icon"
								title="<?php esc_attr_e( 'The description of the category.', 'daext-autolinks-manager' ); ?>"></div>
						</th>
						<th></th>
					</tr>
					</thead>
					<tbody>

					<?php foreach ( $results as $result ) : ?>
						<tr>
							<td><?php echo intval( $result['category_id'], 10 ); ?></td>
							<td><?php echo esc_html( stripslashes( $result['name'] ) ); ?></td>
							<td><?php echo esc_html( stripslashes( $result['description'] ) ); ?></td>
							<td class="icons-container">
								<form method="POST"
										action="admin.php?page=<?php echo esc_attr( $this->shared->get( 'slug' ) ); ?>-categories">
									<?php wp_nonce_field( 'daextam_clone_category_' . intval( $result['category_id'], 10 ), 'daextam_clone_category_nonce' ); ?>
									<input type="hidden" name="clone_id" value="<?php echo esc_attr( $result['category_id'] ); ?>">
									<input class="menu-icon clone help-icon" type="submit" value="">
								</form>
								<a class="menu-icon edit"
									href="admin.php?page=<?php echo esc_attr( $this->shared->get( 'slug' ) ); ?>-categories&edit_id=<?php echo esc_attr( $result['category_id'] ); ?>"></a>
								<form id="form-delete-<?php echo esc_attr( $result['category_id'] ); ?>" method="POST"
										action="admin.php?page=<?php echo esc_attr( $this->shared->get( 'slug' ) ); ?>-categories">
									<?php wp_nonce_field( 'daextam_delete_category_' . intval( $result['category_id'], 10 ), 'daextam_delete_category_nonce' ); ?>
									<input type="hidden" value="<?php echo esc_attr( $result['category_id'] ); ?>"
											name="delete_id">
									<input class="menu-icon delete" type="submit" value="">
								</form>
							</td>
						</tr>
					<?php endforeach; ?>

					</tbody>

				</table>

			</div>

			<!-- Display the pagination -->
			<?php if ( $pag->total_items > 0 ) : ?>
				<div class="daext-tablenav daext-clearfix">
					<div class="daext-tablenav-pages">
						<span class="daext-displaying-num"><?php echo esc_html( $pag->total_items ); ?>
							&nbsp<?php esc_html_e( 'items', 'daext-autolinks-manager' ); ?></span>
						<?php $pag->show(); ?>
					</div>
				</div>
			<?php endif; ?>

		<?php else : ?>

			<?php

			if ( mb_strlen( trim( $filter ) ) > 0 ) {
				echo '<div class="error settings-error notice is-dismissible below-h2"><p>' . esc_html__( 'There are no results that match your filter.', 'daext-autolinks-manager' ) . '</p></div>';
			}

			?>

		<?php endif; ?>

		<div>

			<form method="POST" action="admin.php?page=<?php echo esc_attr( $this->shared->get( 'slug' ) ); ?>-categories"
					autocomplete="off">

				<input type="hidden" value="1" name="form_submitted">
				<?php wp_nonce_field( 'daextam_create_update_category', 'daextam_create_update_category_nonce' ); ?>

				<?php if ( ! is_null( $data['edit_id'] ) ) : ?>

				<!-- Edit a Category -->

				<div class="daext-form-container">

					<h3 class="daext-form-title"><?php esc_html_e( 'Edit Category', 'daext-autolinks-manager' ); ?>&nbsp<?php echo esc_html( $category_obj->category_id ); ?></h3>

					<table class="daext-form daext-form-table">

						<input type="hidden" name="update_id"
								value="<?php echo esc_attr( $category_obj->category_id ); ?>"/>

						<!-- Name -->
						<tr valign="top">
							<th><label for="title"><?php esc_html_e( 'Name', 'daext-autolinks-manager' ); ?></label></th>
							<td>
								<input value="<?php echo esc_attr( stripslashes( $category_obj->name ) ); ?>" type="text"
										id="name" maxlength="100" size="30" name="name"/>
								<div class="help-icon"
									title="<?php esc_attr_e( 'The name of the category.', 'daext-autolinks-manager' ); ?>"></div>
							</td>
						</tr>

						<!-- Description -->
						<tr valign="top">
							<th><label for="title"><?php esc_html_e( 'Description', 'daext-autolinks-manager' ); ?></label></th>
							<td>
								<input value="<?php echo esc_attr( stripslashes( $category_obj->description ) ); ?>"
										type="text" id="description" maxlength="255" size="30" name="description"/>
								<div class="help-icon"
									title="<?php esc_attr_e( 'The description of the category.', 'daext-autolinks-manager' ); ?>"></div>
							</td>
						</tr>

					</table>

					<!-- submit button -->
					<div class="daext-form-action">
						<input class="button" type="submit"
								value="<?php esc_attr_e( 'Update Category', 'daext-autolinks-manager' ); ?>">
						<input id="cancel" class="button" type="submit"
								value="<?php esc_attr_e( 'Cancel', 'daext-autolinks-manager' ); ?>">
					</div>

					<?php else : ?>

					<!-- Create a Category -->

					<div class="daext-form-container">

						<div class="daext-form-title"><?php esc_html_e( 'Create New Category', 'daext-autolinks-manager' ); ?></div>

						<table class="daext-form daext-form-table">

							<!-- Name -->
							<tr valign="top">
								<th><label for="title"><?php esc_html_e( 'Name', 'daext-autolinks-manager' ); ?></label></th>
								<td>
									<input type="text" id="name" maxlength="100" size="30" name="name"/>
									<div class="help-icon"
										title="<?php esc_attr_e( 'The name of the category.', 'daext-autolinks-manager' ); ?>"></div>
								</td>
							</tr>

							<!-- Description -->
							<tr valign="top">
								<th><label for="title"><?php esc_html_e( 'Description', 'daext-autolinks-manager' ); ?></label></th>
								<td>
									<input type="text" id="description" maxlength="255" size="30" name="description"/>
									<div class="help-icon"
										title="<?php esc_attr_e( 'The description of the category.', 'daext-autolinks-manager' ); ?>"></div>
								</td>
							</tr>

						</table>

						<!-- submit button -->
						<div class="daext-form-action">
							<input class="button" type="submit"
									value="<?php esc_attr_e( 'Add Category', 'daext-autolinks-manager' ); ?>">
						</div>

						<?php endif; ?>

					</div>

			</form>

		</div>

	</div>

</div>

<!-- Dialog Confirm -->
<div id="dialog-confirm" title="<?php esc_attr_e( 'Delete the category?', 'daext-autolinks-manager' ); ?>" class="daext-display-none">
	<p><?php esc_html_e( 'This category will be permanently deleted and cannot be recovered. Are you sure?', 'daext-autolinks-manager' ); ?></p>
</div>
