<?php
/**
 * The file used to display the "Options" menu in the admin area.
 *
 * @package daext-autolinks-manager
 */

if ( ! current_user_can( 'manage_options' ) ) {
	wp_die( esc_html__( 'You do not have sufficient capabilities to access this page.', 'daext-autolinks-manager' ) );
}

//Sanitization -------------------------------------------------------------------------------------------------
$data['settings_updated'] = isset( $_GET['settings-updated'] ) ? sanitize_key( $_GET['settings-updated'], 10 ) : null;
$data['active_tab']       = isset( $_GET['tab'] ) ? sanitize_key( $_GET['tab'] ) : 'defaults_options';

?>

<div class="wrap">

	<h2><?php esc_attr_e( 'Autolinks Manager - Options', 'daext-autolinks-manager' ); ?></h2>

	<?php

	// Settings errors.
	if ( ! is_null( $data['settings_updated'] ) && 'true' === $data['settings_updated'] ) {
		settings_errors();
	}

	?>

	<div id="daext-options-wrapper">

		<div class="nav-tab-wrapper">
			<a href="?page=daextam-options&tab=defaults_options"
				class="nav-tab <?php echo 'defaults_options' === $data['active_tab'] ? 'nav-tab-active' : ''; ?>"><?php esc_html_e( 'Defaults', 'daext-autolinks-manager' ); ?></a>
			<a href="?page=daextam-options&tab=analysis_options"
				class="nav-tab <?php echo 'analysis_options' === $data['active_tab'] ? 'nav-tab-active' : ''; ?>"><?php esc_html_e( 'Analysis', 'daext-autolinks-manager' ); ?></a>
			<a href="?page=daextam-options&tab=advanced_options"
				class="nav-tab <?php echo 'advanced_options' === $data['active_tab'] ? 'nav-tab-active' : ''; ?>"><?php esc_html_e( 'Advanced', 'daext-autolinks-manager' ); ?></a>
		</div>

		<form method="post" action="options.php" autocomplete="off">

			<?php

			if ( 'defaults_options' === $data['active_tab'] ) {

				settings_fields( $this->shared->get( 'slug' ) . '_defaults_options' );
				do_settings_sections( $this->shared->get( 'slug' ) . '_defaults_options' );

			}

			if ( 'analysis_options' === $data['active_tab'] ) {

				settings_fields( $this->shared->get( 'slug' ) . '_analysis_options' );
				do_settings_sections( $this->shared->get( 'slug' ) . '_analysis_options' );

			}

			if ( 'advanced_options' === $data['active_tab'] ) {

				settings_fields( $this->shared->get( 'slug' ) . '_advanced_options' );
				do_settings_sections( $this->shared->get( 'slug' ) . '_advanced_options' );

			}

			?>

			<div class="daext-options-action">
				<input type="submit" name="submit" id="submit" class="button"
						value="<?php esc_attr_e( 'Save Changes', 'daext-autolinks-manager' ); ?>">
			</div>

		</form>

	</div>

</div>
